/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
<pre>{@code
                   y
                   |
                   |
                   | v[4]
                 1 +-----------------+ v[5]=(1,1,0)
                  /|                /|
                /  |              /  |
              /    |            /    |
            /      |          /      |
      v[7] +-----------------+ v[6]  |
           |       |         |       |                    y
           |       |         |       |                    |
           |       |         |       | v[1]               |
           |  v[0] +---------|-------+------> x           |
           |      /          |      /1                    |
           |    /            |    /                       +-----> x
           |  /              |  /                        /
           |/                |/                         /
         1 +-----------------+                         /
         / v[3]=(0,0,1)      v[2]=(1,0,1)            z
       /
     /
    z
}</pre>
*/
public class OfflineCubes
{
   public static void main(String[] args)
   {
      // Create Model objects to hold the geometry.
      Model model1 = new Model();
      Model model2 = new Model();

      // Add geometry to the Models
      // First cube
      Vertex v0 = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      Vertex v1 = new Vertex(1.0, 0.0, 0.0);
      Vertex v2 = new Vertex(1.0, 0.0, 1.0);
      Vertex v3 = new Vertex(0.0, 0.0, 1.0);
      Vertex v4 = new Vertex(0.0, 1.0, 0.0);  // four vertices around the top face
      Vertex v5 = new Vertex(1.0, 1.0, 0.0);
      Vertex v6 = new Vertex(1.0, 1.0, 1.0);
      Vertex v7 = new Vertex(0.0, 1.0, 1.0);

      // Second cube.
      Vertex w0 = new Vertex(0.0, 0.0, 0.0);  // four vertices around the bottom face
      Vertex w1 = new Vertex(2.0, 0.0, 0.0);
      Vertex w2 = new Vertex(2.0, 0.0, 2.0);
      Vertex w3 = new Vertex(0.0, 0.0, 2.0);
      Vertex w4 = new Vertex(0.0, 2.0, 0.0);  // four vertices around the top face
      Vertex w5 = new Vertex(2.0, 2.0, 0.0);
      Vertex w6 = new Vertex(2.0, 2.0, 2.0);
      Vertex w7 = new Vertex(0.0, 2.0, 2.0);

      // First cube.
      model1.addLineSegment(v0, v1);  // bottom face
      model1.addLineSegment(v1, v2);  // bottom face
      model1.addLineSegment(v2, v3);  // bottom face
      model1.addLineSegment(v3, v0);  // bottom face
      model1.addLineSegment(v4, v5);  // top face
      model1.addLineSegment(v5, v6);  // top face
      model1.addLineSegment(v6, v7);  // top face
      model1.addLineSegment(v7, v4);  // top face
      model1.addLineSegment(v0, v4);  // back face
      model1.addLineSegment(v1, v5);  // back face
      model1.addLineSegment(v3, v7);  // front face
      model1.addLineSegment(v2, v6);  // front face

      // Second cube.
      model2.addLineSegment(w0, w1);  // bottom face
      model2.addLineSegment(w1, w2);  // bottom face
      model2.addLineSegment(w2, w3);  // bottom face
      model2.addLineSegment(w3, w0);  // bottom face
      model2.addLineSegment(w4, w5);  // top face
      model2.addLineSegment(w5, w6);  // top face
      model2.addLineSegment(w6, w7);  // top face
      model2.addLineSegment(w7, w4);  // top face
      model2.addLineSegment(w0, w4);  // back face
      model2.addLineSegment(w1, w5);  // back face
      model2.addLineSegment(w3, w7);  // front face
      model2.addLineSegment(w2, w6);  // front face


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add the Models to the Scene.
      scene.addModel(model1, model2);

      // Push the models away from where the camera is.
      for (Vertex v : model1.vertexList)
      {
         v.z -= 4;
      }
      for (Vertex v : model2.vertexList)
      {
         v.z -= 4;
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      for (int i = 0; i <= 100; i++)
      {
         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineCubes_Frame%03d.ppm", i));

         // Update the scene by translating in
         // camera space each of the vertices.
         for (Vertex v : model1.vertexList)
         {
            v.x -= 0.1;
            v.y -= 0.03;
          //v.z += 0.02;
         }
         for (Vertex v : model2.vertexList)
         {
            v.x += 0.1;
          //v.y += 0.4;
          //v.z -= 0.01;
         }
      }
   }
}
