/*

*/

import renderer.scene.*;
import renderer.models.GRSModel;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   This program creates an animation by repeatedly saving the framebuffer's image to a file.
   <p>
   You can see the animation by quickly sequencing through the frame images.
*/
public class OfflineGRS
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      final Model model = new GRSModel(new File("../assets/grs/knight.grs"));

      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Push the models away from where the camera is.
      for (Vertex v : model.vertexList)
      {
         v.z -= 1;
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Render our scene into the frame buffer.
      fb.clearFB(Color.darkGray);
      Pipeline.debug  = true;
      Rasterize.debug = false;
      Pipeline.render(scene, fb);
      fb.dumpFB2File(String.format("PPM_OfflineGRS_Frame%03d.ppm", 0));
      Pipeline.debug  = false;
      Rasterize.debug = false;

      for (int i = 1; i <= 60; i++)
      {
         // Update the scene by translating in
         // camera space each of the model's vertices.
         for (Vertex v : model.vertexList)
         {
            v.x += 0.1;
            v.y -= 0.01;
            v.z -= 0.05;
         }

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineGRS_Frame%03d.ppm", i));
      }
   }
}
