/*

*/

import renderer.scene.*;
import renderer.models.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.io.File;
import java.awt.Color;

/**
   This program creates an animation by repeatedly saving the framebuffer's image to a file.
   <p>
   You can see the animation by quickly sequencing through the frame images.
*/
public class OfflineMovie
{
   public static void main(String[] args)
   {
      // Create several Model objects.
      Model m1 = new ObjSimpleModel(new File("../assets/apple.obj"));
      Model m2 = new ObjSimpleModel(new File("../assets/cow.obj"));
      Model m3 = new ObjSimpleModel(new File("../assets/cessna.obj"));
      Model m4 = new ObjSimpleModel(new File("../assets/galleon.obj"));
      Model m5 = new ObjSimpleModel(new File("../assets/teapot.obj"));
      Model m6 = new ObjSimpleModel(new File("../assets/small_rhombicosidodecahedron.obj"));
      Model m7 = new ObjSimpleModel(new File("../assets/great_rhombicosidodecahedron.obj"));


      // Create the Scene object that we shall render
      Scene scene = new Scene();

      // Add Models to the Scene.
      scene.addModel(m1, m3, m4, m2);//apple, cessna, galleon, cow

      // Push the models away from where the camera is.
      for (Model m : scene.modelList)
         for (Vertex v : m.vertexList)
         {
            v.z -= 3;
         }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a framebuffer to render our scene into.
      int vp_width  = 512;
      int vp_height = 512;
      FrameBuffer fb = new FrameBuffer(vp_width, vp_height);
      // Give the framebuffer a nice background color.
      fb.clearFB(Color.darkGray);

      // Render our scene into the frame buffer.
      Pipeline.render(scene, fb);
      // Save the resulting image in a file.
      fb.dumpFB2File( String.format("PPM_OfflineModels_Frame%03d.ppm", 0) );

      for (int i = 1; i <= 100; i++)
      {
         // Update the scene.
         for (Vertex v : scene.modelList.get(0).vertexList)
         {
            v.x += 0.3;  // move right
            v.y += 0.1;  // move up
            v.z -= 0.1;  // move back
         }
         for (Vertex v : scene.modelList.get(1).vertexList)
         {
            v.x -= 0.3;  // move left
            v.y -= 0.1;  // move down
            v.z -= 0.1;  // move back
         }
         for (Vertex v : scene.modelList.get(2).vertexList)
         {
            v.x += 0.01; // move right
         }

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineModels_Frame%03d.ppm", i));
      }
   }
}
