/*

*/

import renderer.scene.*;
import renderer.pipeline.*;
import renderer.framebuffer.*;

import java.awt.Color;

/**
<pre>{@code
                   y
                   |
                   |
                   | v[1]
                 1 +
                   |
                   |
                   |
                   |
                   |
                   |
                   |
                   |                  v[0]
              v[2] +-----------------+------> x
                  /                  1
                /
              /
            /
         1 +
         / v[2]
       /
     /
    z
}</pre>
   Render a single wireframe triangle. This is just about
   the simplest possible model. It is useful for debugging.
*/
public class OfflineTriangle
{
   public static void main(String[] args)
   {
      // Create a Model object to hold the geometry.
      final Model model = new Model();

      // Add geometry to the Model.
      Vertex v0 = new Vertex(1.0, 0.0, 0.0);
      Vertex v1 = new Vertex(0.0, 1.0, 0.0);
      Vertex v2 = new Vertex(0.0, 0.0, 0.0);
    //Vertex v2 = new Vertex(0.0, 0.0, 1.0);  // lands right on the camera

      model.addLineSegment(v1, v2);
      model.addLineSegment(v2, v0);
      model.addLineSegment(v0, v1);

      // Create the Scene object that we shall render.
      Scene scene = new Scene();

      // Add the Model to the Scene.
      scene.addModel(model);

      // Push the model away from where the camera is.
      for (Vertex v : model.vertexList)
      {
         v.z -= 2;
      }

      // Log interesting information to standard output.
      //System.out.println( scene.toString() );

      // Create a FrameBuffer to render our scene into.
      int width  = 512;
      int height = 512;
      FrameBuffer fb = new FrameBuffer(width, height);

      // Render our scene into the frame buffer.
      fb.clearFB(Color.darkGray);
      Pipeline.debug  = true;
      Rasterize.debug = true;
      Pipeline.render(scene, fb);
      fb.dumpFB2File(String.format("PPM_OfflineTriangle_Frame%03d.ppm", 0));
      Pipeline.debug  = false;
      Rasterize.debug = false;

      for (int i = 1; i <= 60; i++)
      {
         // Update the scene by translating in
         // camera space each of the 8 vertices.
         for (Vertex v : model.vertexList)
         {
            v.x += 0.2;
          //v.y -= 0.1;
            v.z -= 0.05;
         }

         // Render again.
         fb.clearFB(Color.darkGray);
         Pipeline.render(scene, fb);
         fb.dumpFB2File(String.format("PPM_OfflineTriangle_Frame%03d.ppm", i));
      }
   }
}
