/*

*/

package renderer.scene;

/**
   A {@code LineSegment} object has two integers that
   represent the endpoints of the line segment. The
   integers are indices into the {@link Vertex} list
   of a {@link Model} object.
*/
/*
   NOTE: The LineSegment class could be an inner class of
         the Model class. Then each LineSegment object would
         automatically have acces to the actual Vertex list
         that the LineSegment is indexing into. Here, we
         have every LineSegment object hold a reference to
         the Model object that it is "part of".
*/
public class LineSegment
{
   public int[] index = new int[2]; // the indices for this line segment
   public Model model;

   /**
      Create a {@code LineSegment} object with two integer indices
      into a {@link Model}'s list of {@link Vertex} objects.

      @param i0     index of 1st endpoint of the new {@code LineSegment}
      @param i1     index of 2nd endpoint of the new {@code LineSegment}
      @param model  {@link Model} containing the {@link Vertex} list for the new {@code LineSegment}
   */
   public LineSegment(int i0, int i1, Model model)
   {
      this.model = model;
      index[0] = i0;
      index[1] = i1;
   }


   /**
      Create a {@code LineSegment} object with the same two indices
      from the given {@code LineSegment} object but referring to
      the {@link Vertex} list from another {@link Model}.

      @param ls     {@code LineSegment} to make a copy of
      @param model  {@link Model} that the new {@code LineSegment} is part of
   */
   public LineSegment(LineSegment ls, Model model) // a "copy constructor"
   {
      this( ls.index[0], ls.index[1], model );
   }


   /**
      For debugging.

      @return {@link String} representation of this {@code LineSegment} object
   */
   @Override
   public String toString()
   {
      return "Line Segment: (" + index[0] + ", " + index[1] + ")\n";
   }
}
